<?php
/*******************************************************************************
*    myMVC - Model-View-Controller for PHP                                     *
*    Copyright (C) 2012 Luca Bertoncello                                       *
*    Hartigstrasse, 12 - 01127 Dresden Deutschland                             *
*    E-Mail: lucabert@lucabert.de, lucabert@lucabert.com                       *
*    http://www.lucabert.de/  http://www.lucabert.com/                         *
*                                                                              *
*    This program is free software; you can redistribute it and/or modify      *
*    it under the terms of the GNU Lesser General Public License as published  *
*    by the Free Software Foundation; version 2.1 of the License.              *
*                                                                              *
*    This program is distributed in the hope that it will be useful,           *
*    but WITHOUT ANY WARRANTY; without even the implied warranty of            *
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
*    GNU Lesser General Public License for more details.                       *
*                                                                              *
*    You should have received a copy of the GNU Lesser General Public License  *
*    along with this program; if not, write to the Free Software               *
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA *
*******************************************************************************/

/**
 * Template engine for Views, using Smarty (http://www.smarty.net/)
 */
class myMVC_SmartyEngine extends myMVC_BaseEngine
{
  private $smarty;

  /**
   * Constructor.
   * 
   * @param string $templateDirectory   The full path of the directory in which the templates are saved
   * @param string $compileDirectory    The full path of a directory used to compile the Smarty-templates.
   *                                    WARNING: it <b>must</b> be writeable from the webserver's user!
   */
  public function __construct($templateDirectory, $compileDirectory)
  {
    parent::__construct($templateDirectory, $compileDirectory);
    $this->smarty = new Smarty;
    $this->smarty->template_dir = $templateDirectory;
    $this->smarty->compile_dir = $compileDirectory;
    $this->smarty->compile_check = true;
  }

  /**
   * Assign a key to the template
   * 
   * @param string $key                 The key
   * @param mixed $value                The value
   */
  public function assignVariable($key, $value)
  {
    $this->smarty->assign($key, $value);
  }

  /**
   * Assign an object to the template (reference)
   * 
   * @param string $key                 The key
   * @param mixed $object               The object
   */
  public function assignObject($key, $object)
  {
    $this->smarty->assign_by_ref($key, $object);
  }

  /**
   * Register a function to be called from template
   * 
   * @param string $name                The name of the function (in template!!)
   * @param mixed $func                 The name function or method (method in the format class::method).
   *                                    Also accepted in the format array('class', 'method').
   */
  public function registerFunction($name, $func)
  {
    if(is_array($func))
      $this->smarty->register_function($name, $func);
    else if(strpos($func, '::') !== false)
      $this->smarty->register_function($name, explode('::', $func));
    else
      $this->smarty->register_function($name, $func);
  }

  /**
   * Register a modifier to be called from template
   * 
   * @param string $name                The name of the modifier (in template!!)
   * @param mixed $func                 The name function or method (method in the format class::method)
   *                                    Also accepted in the format array('class', 'method').
   */
  public function registerModifier($name, $func)
  {
    if(is_array($func))
      $this->smarty->register_modifier($name, $func);
    else if(strpos($func, '::') !== false)
      $this->smarty->register_modifier($name, explode('::', $func));
    else
      $this->smarty->register_modifier($name, $func);
  }

  /**
   * Render a template an send it to the browser
   * 
   * @param class $view                 The current view (to get the template)
   */
  public function display($view)
  {
    $this->smarty->display($view->getTemplateFile());
  }

  /**
   * Render a template an returns it as string
   * 
   * @param class $view                 The current view (to get the template)
   * @return string                     The rendered template
   */
  public function fetch($view)
  {
    return $this->smarty->fetch($view->getTemplateFile());
  }
}
?>
